Attribute VB_Name = "modMRQMIN"
'================================================================
' Descrizione.....: Subroutine MRQMIN
' Nome dei Files..: modMRQMIN.bas
' Data............: 24/02/2002
' Versione........: 1.0 a 32 bits.
' Sistema.........: Visual Basic 6.0 (SP5) sotto Windows NT (SP6)
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://www.flanguasco.org
'                   http://members.xoom.virgilio.it/flanguasco/
'================================================================
'
'   Tradotte dalle Subroutines FORTRAN di:
'           Numerical Recipes in FORTRAN 77
'           The Art of Scientific Computing
'           Second Edition Volume 1 of
'           FORTRAN Numerical Recipes
'           di William H. Press ed altri
'
'   Nota:   Tutti i vettori e le matrici di queste
'           routines iniziano dall' indice 1.
'
Option Explicit
'
Private atry#(), beta#(), da#() ' Vettori di lavoro per MrqMin.  Devono rimanere
                                ' invariati tra le successive chiamate.


Sub gaussj(ByRef a#(), ByVal N&, ByVal Np&, ByRef b#(), ByVal M&, ByVal Mp&)
'
'   Linear equation solution by Gauss-Jordan elimination, equation (2.1.1) above.
'   a(1:n,1:n) is an input matrix stored in an array of physical dimensions np by np.
'   b(1:n,1:m) is an input matrix containing the m right-hand side vectors, stored in
'   an array of physical dimensions np by mp. On output, a(1:n,1:n) is replaced by
'   its matrix inverse, and b(1:n,1:m) is replaced by the corresponding set of
'   solution vectors.
'   Parameter: NMAX is the largest anticipated value of n.
'
    Dim I&, icol&, irow&, J&, K&, L&, LL&
    Dim big#, dum#, pivinv#
'
    ' The integer arrays ipiv, indxr, and indxc are used
    ' for bookkeeping on the pivoting.
    ReDim Indxc&(1 To N), Indxr&(1 To N), Ipiv&(1 To N)
'
    For J = 1 To N
        Ipiv(J) = 0
    Next J
    For I = 1 To N      ' This is the main loop over the columns to be reduced.
        big = 0#
        For J = 1 To N  ' This is the outer loop of the search for a pivot element.
            If (Ipiv(J) <> 1) Then
                For K = 1 To N
                    If (Ipiv(K) = 0) Then
                        If (Abs(a(J, K)) >= big) Then
                            big = Abs(a(J, K))
                            irow = J
                            icol = K
                        End If
                    ElseIf (Ipiv(K) > 1) Then
                        'pause 'singular matrix in gaussj'
                        MsgBox "singular matrix in gaussj"
                        Exit Sub
                    End If
                Next K
            End If
        Next J
        Ipiv(icol) = Ipiv(icol) + 1
        ' We now have the pivot element, so we interchange rows, if needed,
        ' to put the pivot element on the diagonal. The columns are not physically
        ' interchanged, only relabeled:
        ' indxc(i), the column of the ith pivot element, is the ith column that is
        ' reduced, while indxr(i) is the row in which that pivot element was originally
        ' located.
        ' If indxr(i) <> indxc (I) there is an implied column interchange. With this
        ' form of bookkeeping, the solution b 's will end up in the correct order,
        ' and the inverse matrix will be scrambled by columns.
        If (irow <> icol) Then
            For L = 1 To N
                dum = a(irow, L)
                a(irow, L) = a(icol, L)
                a(icol, L) = dum
            Next L
            ' In questa applicazione la sub
            ' gaussj e' sempre usata con M = 1.
'            For L = 1 To M
'                dum = b(irow, L)
'                b(irow, L) = b(icol, L)
'                b(icol, L) = dum
'            Next L
            dum = b(irow)
            b(irow) = b(icol)
            b(icol) = dum
        End If
        Indxr(I) = irow ' We are now ready to divide the pivot row by the pivot
                        ' element, located at irow and icol.
        Indxc(I) = icol
        If (a(icol, icol) = 0#) Then
            'pause 'singular matrix in gaussj'
            MsgBox "singular matrix in gaussj"
            Exit Sub
        End If
        pivinv = 1# / a(icol, icol)
        a(icol, icol) = 1#
        For L = 1 To N
            a(icol, L) = a(icol, L) * pivinv
        Next L
'        For L = 1 To M
'            b(icol, L) = b(icol, L) * pivinv
'        Next L
        b(icol) = b(icol) * pivinv
        For LL = 1 To N ' Next, we reduce the rows...
            If (LL <> icol) Then '...except for the pivot one, of course.
                dum = a(LL, icol)
                a(LL, icol) = 0#
                For L = 1 To N
                    a(LL, L) = a(LL, L) - a(icol, L) * dum
                Next L
'                For L = 1 To M
'                    b(LL, L) = b(LL, L) - b(icol, L) * dum
'                Next L
                b(LL) = b(LL) - b(icol) * dum
            End If
        Next LL
    Next I  ' This is the end of the main loop over columns of the reduction.
'
    For L = N To 1 Step -1  ' It only remains to unscramble the solution in view
                            ' of the column interchanges. We do this by in-
                            ' terchanging pairs of columns in the reverse order
                            ' that the permutation was built up.
        If (Indxr(L) <> Indxc(L)) Then
            For K = 1 To N
                dum = a(K, Indxr(L))
                a(K, Indxr(L)) = a(K, Indxc(L))
                a(K, Indxc(L)) = dum
            Next K
        End If
    Next L
    ' And we are done.
'
'
'
End Sub
Sub covsrt(covar#(), ByVal Npc&, ByVal Ma&, Ia&(), ByVal Mfit&)
'
'   Expand in storage the covariance matrix covar, so as to take into account
'   parameters that are being held fixed. (For the latter, return zero covariances.)
'
    Dim I&, J&, K&
    Dim swap#
'
    For I = Mfit + 1 To Ma
        For J = 1 To I
            covar(I, J) = 0#
            covar(J, I) = 0#
        Next J
    Next I
'
    K = Mfit
    For J = Ma To 1 Step -1
        If (Ia(J) <> 0) Then
            For I = 1 To Ma
                swap = covar(I, K)
                covar(I, K) = covar(I, J)
                covar(I, J) = swap
            Next I
            For I = 1 To Ma
                swap = covar(K, I)
                covar(K, I) = covar(J, I)
                covar(J, I) = swap
            Next I
            K = K - 1
        End If
    Next J
'
'
'
End Sub
Sub RegGauss(ByVal X#, a#(), ByRef Y#, ByRef dYda#(), ByVal Ma&)
'
'   y(x; a) is the sum of ma/3 Gaussians (15.5.16).
'   The amplitude, center, and width of the Gaussians are stored in consecutive
'   locations of a: a(i) = Bk , a(i+1) = Ek , a(i+2) = Gk , k = 1; :::; ma/3.
'
    Dim I&
    Dim arg#, ex#, fac#
'
    Y = 0#
    For I = 1 To Ma - 1 Step 3
        arg = (X - a(I + 1)) / a(I + 2)
        ex = Exp(-arg ^ 2)
        fac = a(I) * ex * 2# * arg
        Y = Y + a(I) * ex
        dYda(I) = ex
        dYda(I + 1) = fac / a(I + 2)
        dYda(I + 2) = fac * arg / a(I + 2)
    Next I
'
'
'
End Sub
Public Sub MRQMIN(X#(), Y#(), Sig#(), ByVal Ndata&, ByRef a#(), Ia&(), ByVal Ma&, _
    ByRef covar#(), ByRef alpha#(), ByVal Nca&, ByRef ChiSq#, ByRef alamda#)
'
'   Levenberg-Marquardt method, attempting to reduce the value (Yi - Y(Xi))
'   of a fit between a set of data points x(1:ndata), y(1:ndata) with individual
'   standard deviations sig(1:ndata), and a nonlinear function dependent on ma
'   coefficients a(1:ma). The input array ia(1:ma) indicates by nonzero entries
'   those components of a that should be fitted for, and by zero entries those
'   components that should be held fixed at their input values. The program returns
'   current best-fit values for the parameters a(1:ma), and (Yi - Y(Xi)) = chisq.
'   The arrays covar(1:nca,1:nca), alpha(1:nca,1:nca) with physical dimension nca
'   ( >= the number of fitted parameters) are used as working space during most
'   iterations. Supply a subroutine funcs(x,a,yfit,dyda,ma) that evaluates the
'   fitting function yfit, and its derivatives dyda with respect to the fitting
'   parameters a at x. On the first call provide an initial guess for the parameters
'   a, and set alamda<0 for initialization (which then sets alamda=.001).
'   If a step succeeds chisq becomes smaller and alamda decreases by a factor of 10.
'   If a step fails alamda grows by a factor of 10. You must call this routine
'   repeatedly until convergence is achieved. Then, make one final call with
'   alamda=0, so that covar(1:ma,1:ma) returns the covariance matrix, and alpha
'   the curvature matrix.
'   (Parameters held fixed will return zero covariances.)
'
'   The arguments alpha, a and chisq must not be modified between calls,
'   nor should alamda be, except to set it to zero for the final call.
'
'   USES covsrt, gaussj, mrqcof
'
    Dim J&, K&, L&
    Static ochisq#, Mfit&
'
    If (alamda < 0#) Then 'Initialization.
        ReDim atry(1 To Ma), beta(1 To Ma), da(1 To Ma)
'
        Mfit = 0
        For J = 1 To Ma
            If (Ia(J) <> 0) Then Mfit = Mfit + 1
        Next J
        alamda = 0.001
        'Call mrqcof(X, Y, sig, ndata, a, ia, ma, alpha, beta, nca, chisq, funcs)
        Call mrqcof(X(), Y(), Sig(), Ndata, a(), Ia(), Ma, alpha(), beta(), Nca, ChiSq)
        ochisq = ChiSq
        For J = 1 To Ma
            atry(J) = a(J)
        Next J
    End If
'
    For J = 1 To Mfit   ' Alter linearized fitting matrix, by augmenting
                        ' diagonal elements.
        For K = 1 To Mfit
            covar(J, K) = alpha(J, K)
        Next K
        covar(J, J) = alpha(J, J) * (1# + alamda)
        da(J) = beta(J)
    Next J
    Call gaussj(covar(), Mfit, Nca, da(), 1, 1)     ' Matrix solution.
    If (alamda = 0#) Then 'Once converged, evaluate covariance matrix.
        Call covsrt(covar(), Nca, Ma, Ia(), Mfit)
        Call covsrt(alpha(), Nca, Ma, Ia(), Mfit)   ' Spread out alpha to its full
                                                    ' size too.
        Exit Sub
    End If
'
    J = 0
    For L = 1 To Ma ' Did the trial succeed?
        If (Ia(L) <> 0) Then
            J = J + 1
            atry(L) = a(L) + da(J)
        End If
    Next L
    'Call mrqcof(X, Y, sig, ndata, atry, ia, ma, covar, da, nca, chisq, funcs)
    Call mrqcof(X(), Y(), Sig(), Ndata, atry(), Ia(), Ma, covar(), da(), Nca, ChiSq)
'
    If (ChiSq < ochisq) Then    ' Success, accept the new solution.
        alamda = 0.1 * alamda
        ochisq = ChiSq
        For J = 1 To Mfit
            For K = 1 To Mfit
                alpha(J, K) = covar(J, K)
            Next K
            beta(J) = da(J)
        Next J
        For L = 1 To Ma
            a(L) = atry(L)
        Next L
    Else                        ' Failure, increase alamda and return.
        alamda = 10# * alamda
        ChiSq = ochisq
    End If
'
'
'
End Sub
Private Sub mrqcof(X#(), Y#(), Sig#(), ByVal Ndata&, ByRef a#(), Ia&(), ByVal Ma&, _
    ByRef alpha#(), ByRef beta#(), ByVal Nalp&, ByRef ChiSq#) ', funcs)
'
'   Used by mrqmin to evaluate the linearized fitting matrix alpha,
'   and vector beta as in (15.5.8), and calculate (Yi - Y(Xi)).
'
    'EXTERNAL funcs
'
    Dim Mfit&, I&, J&, L&, K&, M&
    Dim dy#, sig2i#, wt#, ymod#
'
    ReDim dYda#(1 To Ma)
'
    Mfit = 0
    For J = 1 To Ma
        If (Ia(J) <> 0) Then Mfit = Mfit + 1
    Next J
    For J = 1 To Mfit   ' Initialize (symmetric) alpha, beta.
        For K = 1 To J
            alpha(J, K) = 0#
        Next K
        beta(J) = 0#
    Next J
'
    ChiSq = 0#
    For I = 1 To Ndata  ' Summation loop over all data.
        Funcs X(I), a(), ymod, dYda(), Ma
        sig2i = 1# / (Sig(I) * Sig(I))
        dy = Y(I) - ymod
        J = 0
        For L = 1 To Ma
            If (Ia(L) <> 0) Then
                J = J + 1
                wt = dYda(L) * sig2i
                K = 0
                For M = 1 To L
                    If (Ia(M) <> 0) Then
                        K = K + 1
                        alpha(J, K) = alpha(J, K) + wt * dYda(M)
                    End If
                Next M
                beta(J) = beta(J) + dy * wt
            End If
        Next L
        ChiSq = ChiSq + dy * dy * sig2i ' And find (Yi - Y(Xi)).
    Next I
'
    For J = 2 To Mfit ' Fill in the symmetric side.
        For K = 1 To J - 1
            alpha(K, J) = alpha(J, K)
        Next K
    Next J
'
'
'
End Sub
Private Sub RegSin(ByVal X#, a#(), ByRef Y#, ByRef dYda#(), ByVal Ma&)
'
'   Funzione modello sinusoidale:
'    Y = a(1)*Sin(a(2)*X + a(3))
'    dY/da(1) = Sin(a(2)*X + a(3))
'    dY/da(2) = a(1)*Cos(a(2)*X + a(3))*X
'    dY/da(3) = a(1)*Cos(a(2)*X + a(3))
'
    Y = a(1) * Sin(a(2) * X + a(3))
    dYda(1) = Sin(a(2) * X + a(3))
    dYda(2) = a(1) * Cos(a(2) * X + a(3)) * X
    dYda(3) = a(1) * Cos(a(2) * X + a(3))
'
'
'
End Sub
Private Sub Funcs(ByVal X#, a#(), ByRef Y#, ByRef dYda#(), ByVal Ma&)
'
'   In questo caso le funzioni da minimizzare sono indicate
'   dalla proprieta' Funzione$ del Form frmRegNonLin.
'   Nel caso generale il codice della funzione da minimizzare
'   risiede in questa routine.
'
    Select Case frmRegNonLin.Funzione$
    Case "RegGauss"
        Call RegGauss(X, a(), Y, dYda(), Ma)
    Case "RegSin"
        Call RegSin(X, a(), Y, dYda(), Ma)
    End Select
'
'
'
End Sub
