Attribute VB_Name = "modSimplex"
'================================================================
' Descrizione.....: Subroutine SIMPLEX
' Nome dei Files..: modSimplex.bas, frmSimplexOp.frm
' Data............: 28/7/1999
' Versione........: 1.0 a 32 bits.
' Sistema.........: Visual Basic 6.0 (SP5) sotto Windows NT (SP6)
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://www.flanguasco.org
'                   http://members.xoom.virgilio.it/flanguasco/
'================================================================
'
Option Explicit
'
' Variabili specifiche per questo caso di SIMPLEX:
Private X#()    ' Vettore dei parametri da usare nelle chiamate
                ' alla routine di calcolo della funzione da
                ' minimizzare.
'
'----------------------------------------------------------------------
'
Private VERMAT#()            ' Vettori e Matrici usati da SIMPLEX.
Private CENTRE#()            '    "         "      "         "
Private VERVAL#()            '    "         "      "         "
Private NPAR&                ' Variabili usate da SIMPLEX.
Private NVERT&               '    "         "      "
'
Private Const ALFA# = 1#     ' Coefficiente di Riflessione.
Private Const beta# = 0.5    ' Coefficiente di Contrazione.
Private Const GAMMA# = 2#    ' Coefficiente di Riflessione Estesa.
Private Const DELTA# = 0.5   ' Coefficiente di Contrazione Completa.
Public Function RandU(ByVal V_Min!, ByVal V_Max!) As Single
'
'   Ritorna una variabile casuale reale a distribuzione uniforme.
'   V_Min <= RandU < V_Max:
'
    RandU = V_Min + (V_Max - V_Min) * Rnd
'
'
'
End Function
Private Function VERCALC(ByVal ICALCV&) As Double
'
'   In questo caso le funzioni da minimizzare (somma dei quadrati degli errori)
'   sono indicate dalla proprieta' Funzione$ del Form frmRegNonLin.
'   Nel caso generale il codice della funzione da minimizzare
'   risiede in questa routine.
'
    Dim IP&
'
    ' Prepara il vettore X() dei parametri da valutare:
    For IP = 1 To NPAR
        X(IP) = VERMAT(ICALCV, IP)
    Next IP
'
    ' Routine della funzione da minimizzare valutata con i
    ' parametri contenuti nel vettore X():
    Select Case frmRegNonLin.Funzione$
    Case "RegGauss"
        VERCALC = RegGauss(X())
    Case "RegSin"
        VERCALC = RegSin(X())
    End Select
'
'
'
End Function
Private Function RegSin(X#()) As Double
'
'   ----------------------------------------------------------------
'   Definizione & Calcolo del valore della funzione da minimizzare:
'
'   ---->   RegSin:
'   ---->   Interpolazione sinusoidale  Y = X(1) * Sin(X(2) * X + X(3))
'   ---->   con minimizzazione della somma degli errori al quadrato.
'
    Dim I&, Y#, FVal#, EQ#
'
    If (X(1) <= 0) _
    Or (X(2) < PI / (0.9 * XDati(Nrs))) Or (PI / (0.1 * XDati(Nrs)) < X(2)) Then
        ' L' ampiezza deve essere > 0 e le frequenze
        ' limitate a valori accettabili:
        RegSin = 1E+38
'
    Else
        FVal = 0#
        For I = 1 To Nrs
            Y = X(1) * Sin(X(2) * XDati(I) + X(3))
            EQ = (YDati(I) - Y) ^ 2
            FVal = FVal + EQ
        Next I
        RegSin = FVal
    End If
'
'
'
End Function
Private Function RegGauss(X#()) As Double
'
'   ----------------------------------------------------------------
'   Definizione & Calcolo del valore della funzione da minimizzare:
'
'   ---->   RegGauss:
'   ---->   Interpolazione a distribuzione di probabilita' normale
'   ---->    Y = X(1) * Exp(-((X - X(2)) / X(3)) ^ 2)
'   ---->   con minimizzazione della somma degli errori al quadrato.
'
    Dim I&, Y#, FVal#, EQ#
'
    If X(1) <= 0 Then
        ' L' ampiezza deve essere positiva:
        RegGauss = 1E+38
'
    Else
        FVal = 0#
        For I = 1 To Nrs
            Y = X(1) * Exp(-((XDati(I) - X(2)) / X(3)) ^ 2)
            EQ = (YDati(I) - Y) ^ 2
            FVal = FVal + EQ
        Next I
        RegGauss = FVal
    End If
'
'
'
End Function
Public Sub SIMPLEX(ByRef IBestV&, ByRef IWorstV&, ByRef ValBV#, ByRef ValWV#, _
    ByRef ValDf#, ByRef NCiclo&, ByRef NCC&, ByRef OpS$, ByRef BestP#())
'
'   Program SIMPLEX
'   ---------------
'
'   Versioni FORTRAN:
'   1F) F. Languasco - 3M Italia, Ferrania - 1/6/84
'       da: - "Fitting curves to data: the Simplex algorithm
'             is the answer", di Cacaceci e Cacheris. Byte, May 1984.
'           - "A Simplex method for function minimization"
'             di J. Nelder, R. Mead. The Computer Journal, Jan 1965.
'
'   2F) by F. G. Languasco - Unilever Research, Bromham - 19/Jul/88
'
'   Versione 1.0 del programma in VB 3.0.
'   Tradotto dal FORTRAN il 31/5/1995.
'
'   Versione 1.0 del programma in VB 6.0 del 28/7/1999.
'   La stretta discendenza da un programma FORTRAN (collaudato per
'   anni), scusera' l' uso dei "GoTo", cosi' invisi ai puristi.
'
'   Programma di ricerca del minimo con il metodo "Simplex downhill".
'   Deve essere preceduto, per l' impostazione delle condizioni
'   iniziali, da una chiamata ad un' opportuna versione di SIMPLES.
'
'   Definizioni & Dati (vedi SchemaMatrici):
'   ----------------------------------------
'
'   VERMAT#(0 To V&, 1 To P&) > Matrice dei vertici: P e' il numero di
'                               parametri della funzione che si vuole
'                               minimizzare con una opportuna scelta
'                               dei loro valori.
'                               V = P + 1 e' il mumero di vertici, a P
'                               dimensioni, del simplesso.
'                               Il vertice 0 (IV0) e' usato come memoria
'                               temporanea di un Set di parametri
'                               risultanti da una operazione di ricerca.
'
'   VERVAL#(0 To V&)          > Vettore dei valori della funzione da
'                               minimizzare calcolati ai V vertici.
'
'   CENTRE#(1 To P&)          > Vettore dei P valori medi dei parametri
'                               calcolati su V - 1 vertici:
'                               il vertice escluso e' IWORSTV.
'
'   VERCALC#(IV&)               Routine di calcolo della funzione da
'                               minimizzare usando i parametri cor-
'                               rispondenti al vertice IV.
'
'   Valori ritornati:
'   -----------------
'
'   IBESTV&                   > Numero del vertice migliore.
'   IWORSTV&                  > Numero del vertice peggiore.
'
'   ValBV#                    > Valore della funzione da minimizzare
'                               calcolato al vertice migliore.
'   ValWV#                    > Valore della funzione da minimizzare
'                               calcolato al vertice peggiore.
'   ValDf#                    > Differenza tra i due valori precedenti.
'
'   NCiclo&                   > Numero del ciclo di ricerca corrente.
'   NCC&                      > Numero di operazioni consecutive di
'                               "Contrazione Completa".
'   OpS$                      > Nome dell' operazione effettuata nel
'                               ciclo corrente.
'
'   BestP#(1 To P&)           > Vettore dei migliori parametri trovati.
'
'----------------------------------------------------------------------------
'
    Dim IV&, IP&, IBP&, CENVAL#
'
'   Algoritmo 'SIMPLEX' di ricerca del Minimo
'   -----------------------------------------
'
    NCiclo = NCiclo + 1
'
    ' Calcola il valor medio dei parametri:
    For IP = 1 To NPAR
        CENVAL = 0#
        For IV = 1 To NVERT
            If IV <> IWorstV Then CENVAL = CENVAL + VERMAT(IV, IP)
        Next IV
        CENTRE(IP) = CENVAL / CDbl(NVERT - 1)
    Next IP
'
'
O400:   ' Riflessione:
    For IP = 1 To NPAR
        VERMAT(0, IP) = (1# + ALFA) * CENTRE(IP) _
                      - ALFA * VERMAT(IWorstV, IP)
    Next IP
    VERVAL(0) = VERCALC(0)
    If (VERVAL(0) >= VERVAL(IWorstV)) Then GoTo O600  ' -> Contrazione.
    NCC = 0
    OpS$ = "Riflessione"
    If (VERVAL(0) >= VERVAL(IBestV)) Then GoTo O800   ' -> Sostituzione
                                                      ' di IWORSTV con
                                                      ' IV0.
'
'
O500:   ' Riflessione Estesa:
    For IP = 1 To NPAR
        VERMAT(IWorstV, IP) = VERMAT(0, IP)
        VERMAT(0, IP) = (1# - GAMMA) * CENTRE(IP) _
                      + GAMMA * VERMAT(IWorstV, IP)
    Next IP
    VERVAL(IWorstV) = VERVAL(0)
    VERVAL(0) = VERCALC(0)
    If (VERVAL(0) >= VERVAL(IBestV)) Then GoTo O900   ' -> Exit Sub.
    OpS$ = "Riflessione Estesa"
    GoTo O800   ' -> Sostituzione di IWORSTV con IV0.
'
'
O600:   ' Contrazione:
    For IP = 1 To NPAR
        VERMAT(0, IP) = (1# - beta) * CENTRE(IP) _
                      + beta * VERMAT(IWorstV, IP)
    Next IP
    VERVAL(0) = VERCALC(0)
    If (VERVAL(0) >= VERVAL(IWorstV)) Then GoTo O700  ' -> Contrazione
                                                      ' Completa.
    NCC = 0
    OpS$ = "Contrazione"
'
'
O800:   ' Sostituzione di IWORSTV con IV0:
    For IP = 1 To NPAR
        VERMAT(IWorstV, IP) = VERMAT(0, IP)
    Next IP
    VERVAL(IWorstV) = VERVAL(0)
    GoTo O900   ' -> Exit Sub.
'
'
O700:   ' Contrazione Completa:
    For IV = 1 To NVERT
        If IV <> IBestV Then
            For IP = 1 To NPAR
                VERMAT(IV, IP) = (1# - DELTA) * VERMAT(IBestV, IP) _
                               + DELTA * VERMAT(IV, IP)
            Next IP
            VERVAL(IV) = VERCALC(IV)
        End If
    Next IV
    NCC = NCC + 1
    OpS$ = "Contrazione Completa"
'
'
O900:   ' Ricerca dei migliori e peggiori valori correnti:
    ValBV = VERVAL(1)
    ValWV = VERVAL(1)
    IBestV = 1
    IWorstV = 1
    For IV = 2 To NVERT
        If VERVAL(IV) < ValBV Then
            ValBV = VERVAL(IV)
            IBestV = IV
        End If
        If VERVAL(IV) > ValWV Then
            ValWV = VERVAL(IV)
            IWorstV = IV
        End If
    Next IV
    ValDf = ValWV - ValBV
'
    ' Ritorna i parametri del vertice migliore:
    IP = 0
    For IBP = LBound(BestP) To UBound(BestP)
        IP = IP + 1
        BestP(IBP) = VERMAT(IBestV, IP)
    Next IBP
'
'
'
End Sub
Public Sub SIMPLES(ByRef IBestV&, ByRef IWorstV&, _
    ByRef ValBV#, ByRef ValWV#, ByRef ValDf#, _
    ByVal Np&, ByVal FPIn As Boolean, ParIn#())
'
'   Imposta le condizioni iniziali per le operazioni di SIMPLEX.
'   NP& e' il numero di parametri della funzione da minimizzare.
'   Ritorna i valori correnti di IBestV&, IWorstV&, ValBV#, ValWV#
'   e ValDf# da usare nelle chiamate successive di SIMPLEX.
'   Se FPIn = True, usa i dati del vettore ParIn#() per impostare
'   un valore iniziale, non casuale, dei parametri da trovare.
'   Le funzioni RandN e RandU sono di ausilio alla generazione
'   dei parametri.
'
    Dim IV&, IP&, IPIn&, Per#
    ReDim X#(1 To Np)
'
    NPAR = Np
    NVERT = NPAR + 1
'
    ReDim VERMAT(0 To NVERT, 1 To NPAR)
    ReDim VERVAL(0 To NVERT)
    ReDim CENTRE(1 To NPAR)
'
    Randomize 'Timer
    For IV = 1 To NVERT
        IPIn = LBound(ParIn)
        For IP = 1 To NPAR
            If FPIn Then
                If IV = 1 Then
                    ' Nel vertice 1 vengono messi i parametri
                    ' scelti come iniziali:
                    VERMAT(IV, IP) = ParIn(IPIn)
                Else
                    ' Negli altri vertici i parametri possono essere
                    ' funzione del vertice 1:
                    Per = ParIn(IPIn) * 10#
                    VERMAT(IV, IP) = ParIn(IPIn) + RandU(-Per, Per)
                    'VERMAT(IV, IP) = RandN(CSng(ParIn(IPIn)), Per)
'
                    ' oppure scelti in modo casuale:
                    'VERMAT(IV, IP) = RandU(-100, 100)
                    'VERMAT(IV, IP) = RandU(1, 10)
                End If
                IPIn = IPIn + 1
            Else
                ' I parametri vengono scelti in modo casuale
                ' in un intervallo appropriato da determinare,
                ' caso per caso, per una miglior convergenza:
                VERMAT(IV, IP) = RandU(-10, 10)
            End If
        Next IP
        VERVAL(IV) = VERCALC(IV)
    Next IV
'
    ' Ricerca dei migliori e peggiori valori correnti
    ' da passare alle chiamate successive di SIMPLEX:
    ValBV = VERVAL(1)
    ValWV = VERVAL(1)
    IBestV = 1
    IWorstV = 1
    For IV = 2 To NVERT
        If ValBV > VERVAL(IV) Then
            ValBV = VERVAL(IV)
            IBestV = IV
        End If
        If ValWV < VERVAL(IV) Then
            ValWV = VERVAL(IV)
            IWorstV = IV
        End If
    Next IV
    ValDf = ValWV - ValBV
'
'
'
End Sub
Public Function RandN(ByVal VMedio!, ByVal Sigma!) As Single
'
'   Ritorna una variabile casuale reale a distribuzione normale.
'   VMedio - 6 * Sigma <= RandN < VMedio + 6 * Sigma:
'
    Dim E!, I&
'
    E = 0!
    For I = 1 To 12
        E = E + Rnd
    Next I
'
    RandN = VMedio + Sigma * (E - 6!)
'
'
'
End Function
Private Sub SchemaMatrici()
'
'
'   Definizioni:
'   m             = Numero di parametri della funzione da minimizzare.
'   n             = m + 1 Numero di vertici del simplesso.
'   f(P1, .., Pm) = Funzione da minimizzare a m parametri.
'
'   Parametri:
'   P1, P2, .., Pm          sono le righe di VERMAT() e gli elementi
'                           di CENTRE().
'   Vertici:
'   V0, V1, V2, V3, .., Vn  sono le colonne di VERMAT() e gli elementi
'                           di VERVAL().
'
'   Matrici e Vettori:
'   VERMAT(0 To n, 1 To m):
'        V0   V1   V2   V3   ..   Vn
'       ------------------------------
'   P1 | Px1  P11  P21  P31  ...  Pn1
'   P2 | Px2  P12  P22  P32  ...  Pn2
'   .. | ...  ...  ...  ...  ...  ...
'   Pm | Pxm  P1m  P2m  P3m  ...  Pnm
'                   ^
' Se, e.g., V2 = IWORSTV = vertice da scartare nel calcolo di CENTRE(P):
'
'   CENTRE(1 To m):
'   P1 | (P11 + P31 + ..., + Pn1)/(n - 1)
'   P2 | (P12 + P32 + ..., + Pn2)/(n - 1)
'   .. | (... + ... + ..., + ...)/(n - 1)
'   Pm | (P1m + P3m + ..., + Pnm)/(n - 1)
'
'
'   VERVAL(0 To n):
'   V0 | f(Px1, ..., Pxm)  = VERCALC(0)
'   V1 | f(P11, ..., P1m)  = VERCALC(1)
'   V2 | f(P21, ..., P2m)  = VERCALC(2)
'   V3 | f(P31, ..., P3m)  = VERCALC(3)
'   .. | f(..., ..., ...)  = VERCALC(.)
'   Vn | f(Pn1, ..., Pnm)  = VERCALC(n)
'
'
'
End Sub
