Attribute VB_Name = "modUtilita"
'================================================================
' Descrizione.....: Variabili, costanti e funzioni di utilita'
'                   per le regressioni con modello non lineare.
' Nome dei Files..: modUtilita.bas
' Data............: 22/02/2004
' Versione........: 1.0 a 32 bits.
' Sistema.........: Visual Basic 6.0 (SP5) sotto Windows NT (SP6)
' Scritto da......: F. Languasco 
' E-Mail..........: MC7061@mclink.it
' DownLoads a.....: http://www.flanguasco.org
'                   http://members.xoom.virgilio.it/flanguasco/
'================================================================
Option Explicit
'
Public Const Nrs& = 200     ' N di punti dati.
Public XDati#(1 To Nrs)     ' Vettori delle ascisse
Public YDati#(1 To Nrs)     ' e delle ordinate dei punti dati.
'
Public Const PI# = 3.14159265358979    ' 4# * Atn(1#)
Public Function Asin(ByVal X As Double) As Double
'
'   Calcola l' ArcoSeno di x con -1 <= x <= 1.
'   Asin e' in radianti.
'
    If Abs(X) < 1# Then
        Asin = Atn(X / Sqr(1# - X * X))
    ElseIf X = 1# Then
        Asin = PI / 2#
    ElseIf X = -1# Then
        Asin = -PI / 2#
    Else
        Err.Raise 5, "Asin Function", "Argomento illegale per Asin"
    End If
'
'
'
End Function
Public Function DMAXVAL(dVet#()) As Double
'
'   Ritorna il valore massimo contenuto nel vettore dVet().
'   Implementa l' equivalente funzione del FORTRAN.
'
    Dim J&, J1&, J2&, dDMax#
'
    J1 = LBound(dVet)
    J2 = UBound(dVet)
    dDMax = dVet(J1)
    For J = J1 + 1 To J2
        If dDMax < dVet(J) Then dDMax = dVet(J)
    Next J
'
    DMAXVAL = dDMax
'
'
'
End Function
Public Function DMINVAL(dVet#()) As Double
'
'   Ritorna il valore minimo contenuto nel vettore dVet().
'   Implementa l' equivalente funzione del FORTRAN.
'
    Dim J&, J1&, J2&, dDMin#
'
    J1 = LBound(dVet)
    J2 = UBound(dVet)
    dDMin = dVet(J1)
    For J = J1 + 1 To J2
        If dDMin > dVet(J) Then dDMin = dVet(J)
    Next J
'
    DMINVAL = dDMin
'
'
'
End Function
Public Function Quadro(ByVal Foglio As PictureBox, _
    ByVal X0#, ByVal Xn#, ByVal Y0#, ByVal Yn#, _
    Optional ByVal FormatVX$ = "#0.0##", _
    Optional ByVal FormatVY$ = "#0.0##", _
    Optional ByVal Npx& = 1, Optional ByRef PxN_X!, Optional ByRef PxN_Y!, _
    Optional ByVal Titolo$ = "", _
    Optional ByVal UnitaX$ = "", _
    Optional ByVal UnitaY$ = "", _
    Optional ByVal AutoRed As Boolean = False, _
    Optional ByRef SxMin!, Optional ByRef SxMax!, _
    Optional ByRef SyMin!, Optional ByRef SyMax!, _
    Optional ByVal RifCol& = vbGreen, _
    Optional ByVal bGriglia As Boolean = True, _
    Optional ByRef tEsec!) As Boolean
'
'   Routine, di uso generale, per la scalatura di un foglio
'   adatto a rappresentare un grafico y = f(x).
'    Parametri in ingresso:
'     Foglio:    PictureBox da scalare.
'     X0:        Valore minimo di ascissa da rappresentare.
'     Xn:        Valore massimo di ascissa da rappresentare.
'                Deve essere X0 <= Xn.
'     Y0:        Valore minimo di ordinata da rappresentare.
'     Yn:        Valore massimo di ordinata da rappresentare.
'                Deve essere Y0 <= Yn.
'     FormatVX$: Stringa di formato dei valori sull' asse X.
'     FormatVY$: Stringa di formato dei valori sull' asse Y.
'     Npx:       N di Pixels di cui si vuole conoscere
'                larghezza ed altezza in [vbUser].
'     Titolo$:   Titolo del grafico.
'     UnitaX$:   Unita' (o titolo) dell' asse X.
'     UnitaY$:   Unita' (o titolo) dell' asse Y.
'     AutoRed:   Stato di Foglio.AutoRedraw dopo il disegno del quadro.
'     RifCol:    Colore degli assi, dei valori e delle griglie di riferimento.
'     bGriglia:  Se True viene disegnata la griglia di riferimento:
'                se False viene fatta solo la scalatura di Foglio.
'    Parametri in uscita:
'     SxMin:     Valore minimo di ascissa rappresentata.
'     SxMax:     Valore massimo di ascissa rappresentata.
'     SyMin:     Valore minimo di ordinata rappresentata.
'     SyMax:     Valore massimo di ordinata rappresentata.
'     PxN_X:     Larghezza in [vbUser] ed
'     PxN_Y:     altezza in [vbUser] di Npx [Pixels].
'     tEsec:     tempo impiegato a disegnare il quadro [s].
'
'   Rev. 25/03/2003
'
    Dim I&, XI!, D_X!, rrx!, YI!, D_Y!, rry!, TxW!, Txt$
    Dim QxMin!, QxMax!, QyMin!, QyMax!
'
    tEsec = Timer
    On Error GoTo Quadro_ERR
'
    ' Verifico la correttezza delle scale:
    If X0 > Xn Then Err.Raise 1001, "Quadro", "Errore di scala X."
    If Y0 > Yn Then Err.Raise 1001, "Quadro", "Errore di scala Y."
'
'-------------------------------------------------------------------------------------
'   Calcolo del passo di grigliatura degli assi.
'
    Dim DYMin!
    Const ScRis& = 5                ' La risoluzione delle scale e' 10 ^ -ScRis.
    Const ScMin! = 10 ^ (1 - ScRis) ' Ampiezza min. delle scale X e Y.
    Const Log10! = 2.30258509299405 ' Log(10#)
'
    ' Imposto una scala minima
    ' per l' asse X:
    If Xn - X0 < ScMin Then
        X0 = X0 - ScMin / 2!
        Xn = Xn + ScMin / 2!
    End If
'
    ' Calcolo la spaziatura dei valori scritti
    ' sull' asse X: la sequenza e' 1, 2, 2.5 e 5:
    D_X = Xn - X0
    rrx = 10! ^ Ceil(Log(D_X / 20!) / Log10)
    Do While D_X / rrx < 5!
        rrx = rrx / 2!
    Loop
    If D_X / rrx > 10! Then rrx = rrx * 2!
    X0 = rrx * Int(Round(X0 / rrx, ScRis))
    Xn = rrx * Ceil(Round(Xn / rrx, ScRis))
    D_X = Xn - X0
    SxMin = X0
    SxMax = Xn
'
    ' Imposto una scala minima
    ' per l' asse Y:
    If Format$(Y0, FormatVY$) = Format$(Yn, FormatVY$) Then
        DYMin = 0.0001
        Do While Format$(Y0 - DYMin, FormatVY$) = Format$(Yn + DYMin, FormatVY$)
            DYMin = 2! * DYMin
        Loop
'
        Y0 = Y0 - 10! * DYMin
        Yn = Yn + 10! * DYMin
    End If
'
    ' Calcolo la spaziatura dei valori scritti
    ' sull' asse Y: la sequenza e' 1, 2, 2.5 e 5:
    D_Y = Yn - Y0
    rry = 10! ^ Ceil(Log(D_Y / 20!) / Log10)
    Do While D_Y / rry < 5!
        rry = rry / 2!
    Loop
    If D_Y / rry > 10! Then rry = rry * 2!
    Y0 = rry * Int(Round(Y0 / rry, ScRis))
    Yn = rry * Ceil(Round(Yn / rry, ScRis))
    D_Y = Yn - Y0
    SyMin = Y0
    SyMax = Yn
'
'-------------------------------------------------------------------------------------
'   Calcola larghezza ed altezza dei bordi.
'
    Dim Bl!, Br!, Bb!, Bt!, BDen!
    Dim TxWs!, TxWd!, TxHt!, TxHb!
'
    ' Imposto i dati di Font dei valori
    ' degli assi:
    Foglio.FontName = "MS Sans Serif"
    Foglio.FontSize = 8
    Foglio.FontBold = False
'
    ' Annullo le scale precedenti:
    Foglio.ScaleMode = vbPixels
'
    ' Il bordo a sinistra deve essere sufficiente
    ' a contenere il valore Y piu' largo:
    TxWs = Foglio.TextWidth(Format$(Y0, FormatVY$) & "W")
    If TxWs < Foglio.TextWidth(Format$(Yn, FormatVY$) & "W") Then
        TxWs = Foglio.TextWidth(Format$(Yn, FormatVY$) & "W")
    End If
'
    ' Il bordo a destra deve essere sufficiente
    ' a contenere il valore Xn e l' etichetta UnitaX$:
    TxWd = (Foglio.TextWidth(Format$(Xn, FormatVX$)) / 2!) _
         + Foglio.TextWidth(UnitaX$ & "W")
'
    ' I bordi a sinistra ed a destra sono:
    BDen = D_X / (Foglio.ScaleWidth - TxWs - TxWd)
    Bl = TxWs * BDen
    Br = TxWd * BDen
'
    ' Il bordo sotto e' 2 volte l' altezza dei valori:
    TxHb = 2! * Abs(Foglio.TextHeight("H"))
'
    ' Il bordo sopra e' 2 volte l' altezza dei valori
    ' piu' l' altezza del titolo:
    Foglio.FontSize = 12
    TxHt = TxHb + Abs(Foglio.TextHeight(Titolo$))
'
    ' I bordi sopra e sotto sono:
    BDen = D_Y / (Abs(Foglio.ScaleHeight) + TxHb + TxHt)
    Bb = TxHb * BDen
    Bt = TxHt * BDen
'
'-------------------------------------------------------------------------------------
'   Imposta la scala e calcola i valori comuni.
'
    Dim Po4_X!, Po4_Y!, EstAx!, EstAy!
'
    ' Imposto i bordi orizzontali
    ' e verticali:
    QxMin = X0 - Bl
    QxMax = X0 + D_X + Br
    QyMin = Y0 - Bb
    QyMax = Y0 + D_Y + Bt
'
    ' La scalatura deve essere permanente:
    Foglio.AutoRedraw = True
'
    ' Imposto la scala e cancello il Foglio:
    Foglio.Scale (QxMin, QyMax)-(QxMax, QyMin)
    Foglio.Line (QxMin, QyMin)-(QxMax, QyMax), Foglio.BackColor, BF ' Questo e' piu' veloce
                                                                    ' di Foglio.Cls.
'
    ' Calcolo larghezza ed altezza di Npx pixels:
    PxN_X = Abs(Foglio.ScaleX(Npx, vbPixels, vbUser))
    PxN_Y = Abs(Foglio.ScaleY(Npx, vbPixels, vbUser))
    ' Calcolo larghezza ed altezza di 4 points:
    Po4_X = Foglio.ScaleX(4, vbPoints, vbUser)  ' Uso i [Points] perche' voglio dimensioni
    Po4_Y = Foglio.ScaleY(4, vbPoints, vbUser)  ' proporzionali a quelle dei caratteri.
'
    ' Estremita' degli assi X e Y:
    EstAx = (Xn + QxMax) / 2!
    EstAy = (Yn + QyMax) / 2!
'
    ' Verifico se e' richiesto il
    ' disegno della griglia:
    If Not bGriglia Then GoTo Fine
'
'-------------------------------------------------------------------------------------
'   Disegna assi, griglie e scrive i valori di scala.
'
    Foglio.FontSize = 8
    Foglio.DrawWidth = 1
    Foglio.DrawMode = vbCopyPen
    Foglio.ForeColor = RifCol
'
    ' Traccio la griglia verticale e scrivo
    ' i valori dell' asse X:
    Foglio.DrawStyle = vbDash
    For XI = X0 To Xn + 0.1 * rrx Step rrx
        Foglio.Line (XI, Y0)-(XI, Yn), RifCol
        Txt$ = Format$(XI, FormatVX$)
        ' Verifico che il formato scelto non
        ' induca ad errori di rappresentazione:
        If Abs(XI - Val(Txt$)) < rrx / 10 Then
            Foglio.CurrentX = XI - Foglio.TextWidth(Txt$) / 2!
            Foglio.CurrentY = Y0 + Po4_Y / 2!
            Foglio.Print Txt$;
        End If
    Next XI
    ' Scrivo l' etichetta dell' asse X:
    If Len(UnitaX$) > 0 Then
        ' Etichetta tutta a destra:
        'Foglio.CurrentX = QxMax - Foglio.TextWidth(UnitaX$ & " ")
        ' Etichetta in centro tra l' ultimo valore ed il bordo a destra:
        Foglio.CurrentX = (Foglio.CurrentX + QxMax - Foglio.TextWidth(UnitaX$)) / 2!
        Foglio.CurrentY = Y0 + Po4_Y / 2!
        Foglio.Print UnitaX$;
    End If
    ' Traccio l' asse Y:
    If (X0 <= 0!) And (0! <= Xn) Then
        Foglio.DrawStyle = vbSolid
        Foglio.Line (0!, Y0)-(0!, EstAy), RifCol
        Foglio.Line (0!, EstAy)-(-Po4_X / 2!, Po4_Y + EstAy), RifCol
        Foglio.Line (0!, EstAy)-(Po4_X / 2!, Po4_Y + EstAy), RifCol
    End If
'
    ' Traccio la griglia orizzontale e scrivo
    ' i valori dell' asse Y:
    Foglio.DrawStyle = vbDash
    For YI = Y0 To Yn + 0.1 * rry Step rry
        Foglio.Line (X0, YI)-(Xn, YI), RifCol
        Txt$ = Format$(YI, FormatVY$)
        ' Verifico che il formato scelto non
        ' induca ad errori di rappresentazione:
        'If Abs(YI - Val(Txt$)) < rry / 10 Then
            Foglio.CurrentX = QxMin
            Foglio.CurrentY = YI - Foglio.TextHeight(Txt$) / 2!
            Foglio.Print Txt$;
        'End If
     Next YI
    ' Scrivo l' etichetta dell' asse Y:
    If Len(UnitaY$) > 0 Then
        Foglio.CurrentX = QxMin
        Foglio.CurrentY = QyMax
        Foglio.Print UnitaY$;
    End If
    ' Traccio l' asse X:
    If (Y0 <= 0!) And (0! <= Yn) Then
        Foglio.DrawStyle = vbSolid
        Foglio.Line (X0, 0!)-(EstAx, 0!), RifCol
        Foglio.Line (EstAx, 0!)-(EstAx - Po4_X, -Po4_Y / 2!), RifCol
        Foglio.Line (EstAx, 0!)-(EstAx - Po4_X, Po4_Y / 2!), RifCol
    End If
'
'-------------------------------------------------------------------------------------
'   Scrive il titolo del grafico:
'
    Dim TitL!, TitT!, TitW!, TitH!
'
    If Len(Titolo$) > 0 Then
        Foglio.FontSize = 12
        Foglio.FontBold = True
        Foglio.ForeColor = vbRed
'
        TitW = Foglio.TextWidth(Titolo$)
        TitH = Foglio.TextHeight(Titolo$)
        ' Verifico che il titolo stia tutto nel Foglio:
        If TitW <= Foglio.ScaleWidth Then
            TitL = (QxMin + QxMax - TitW) / 2!
        ' e se no' lo taglio:
        Else
            TitL = Foglio.ScaleLeft
            Txt$ = " . . . ."
            Titolo$ = Left$(Titolo$, Int(Len(Titolo$) * _
            (Foglio.ScaleWidth - Foglio.TextWidth(Txt$)) / TitW)) & Txt$
        End If
        TitT = QyMax
        ' Cancello l' area su cui andra' scritto il titolo:
        'Foglio.Line (TitL, TitT)-(TitL + TitW, TitT + TitH), Foglio.BackColor, BF
        Foglio.CurrentX = TitL
        Foglio.CurrentY = TitT
        Foglio.Print Titolo$
    End If
'
Fine:
    Foglio.DrawStyle = vbSolid
    Foglio.AutoRedraw = AutoRed
'
'
Quadro_ERR:
    Quadro = (Err = 0)
    If Err <> 0 Then
        MsgBox Err.Description, vbCritical, " Quadro/" & Err.Source
    End If
    tEsec = Timer - tEsec
'
'
'
End Function
Public Function Ceil(ByVal X As Double) As Double
'
'   Funzione di arrotondamento, per numeri reali,
'   all' intero uguale o immediatamente superiore:
'
    If X = Int(X) Then
        Ceil = X
    Else
        Ceil = Int(X) + 1#
    End If
'
'
'
End Function
Public Function IsLoaded(ByVal frmForm As Form) As Boolean
'
'   Ritorna True se il Form frmForm e' gia' caricato in memoria.
'
    Dim X&
'
    For X = 0 To Forms.Count - 1
        If Forms(X) Is frmForm Then
            IsLoaded = True
            Exit Function
        End If
    Next X
'
    IsLoaded = False
'
'
'
End Function
Public Function DMIN1(ParamArray vD() As Variant) As Double
'
'   Implementa la funzione DMIN1(D1, D2, ...DN) del FORTRAN:
'
    Dim J&, J1&, J2&, vDMin As Variant
'
    J1 = LBound(vD)
    J2 = UBound(vD)
    vDMin = vD(J1)
    For J = J1 + 1 To J2
        If vDMin > vD(J) Then vDMin = vD(J)
    Next J
'
    DMIN1 = CDbl(vDMin)
'
'
'
End Function
Public Function SIGN(ByVal dV#, ByVal dS#) As Double
'
'   Ritorna il valore assoluto di dV con il segno di dS.
'   Implementa la funzione SIGN del FORTRAN:
'
    If dS < 0 Then
        SIGN = -Abs(dV)
    Else
        SIGN = Abs(dV)
    End If
'
'
'
End Function
